@@ -35,6 +35,17 @@ class Conclusion(TypedDict):
 
 
 class Querier(Chain):
+    """Queriers are responsible for answering questions about the dataset,
+    using the pre-populated model.
+
+    The high level flow is as follows:
+    1. Determine a set of sub-questions necessary to answer the original.
+    2. For each sub-question, determine a set of queries that would render relevant facts.
+    3. For each query, search the vector store for facts _or_ queries that are similar to the embedded query.
+    4. Extract the facts from these results.
+    5. Recursively summarize up the tree until the original question is answered.
+    """
+
     FACT_PROMPT = PromptTemplate(
         input_variables=["fact", "context", "attributes"],
         template=dedent(
@@ -57,6 +68,8 @@ class Querier(Chain):
     # Questions
 
     def steps_template(self):
+        """The template to determine sub-questions."""
+
         return PromptTemplate(
             template=dedent(
                 """
@@ -77,11 +90,13 @@ class Querier(Chain):
         )
 
     def queries_template(self):
+        """The template to determine sub-question queries."""
+
         return PromptTemplate(
             template=dedent(
                 """
-                Your task is to determine a set of queries which would answer a question.
-                The queries run against a database of facts compiled across several user interviews.
+                Your task is to determine a set of natural-language queries which would answer a question.
+                The queries run against a knowledgebase of facts compiled across several user interviews.
 
                 The overall question you are trying to answer is: {query}
                 You are on the following step: {step}
@@ -97,6 +112,8 @@ class Querier(Chain):
     # Answers
 
     def facts_template(self, facts: list[Fact]):
+        """The template to summarize sub-question queries."""
+
         return FewShotPromptTemplate(
             examples=cast(list[dict], facts),
             example_prompt=self.FACT_PROMPT,
@@ -104,6 +121,7 @@ class Querier(Chain):
                 """
                 Your task is to answer a query against a corpus of user interviews.
                 To help answer the question, you are provided with a set of facts (along with the context and attributes of the author of the fact).
+                If it is not possible to answer, say that you do not know the answer.
 
                 The query is: {query}
 
@@ -116,6 +134,8 @@ class Querier(Chain):
         )
 
     def answers_template(self, answers: list[Answer]):
+        """The template to summarize sub-questions."""
+
         return FewShotPromptTemplate(
             examples=cast(list[dict], answers),
             example_prompt=PromptTemplate(
@@ -145,6 +165,8 @@ class Querier(Chain):
         )
 
     def conclusions_template(self, conclusions: list[Conclusion]):
+        """The template to summarize the final answer."""
+
         return FewShotPromptTemplate(
             examples=cast(list[dict], conclusions),
             example_prompt=PromptTemplate(
@@ -162,6 +184,7 @@ class Querier(Chain):
             prefix=dedent(
                 """
                 Your task is to take a set of steps that were conducted to answer a question, and use them to answer that question.
+                The final answer should be concise and accurate. First provide exactly the answer to the original question, with no extra information. You can then add references, sources, or details.
 
                 The question you are trying to answer: {query}
 
@@ -215,18 +238,15 @@ class Querier(Chain):
             if e
         ]
 
-        new_facts = [f for f in facts if f["fact"] not in self.facts]
-        old_facts = [f for f in facts if f["fact"] in self.facts]
-        facts = (new_facts + old_facts)[:n]
+        new_facts = {f["fact"]: f for f in facts if f["fact"] not in self.facts}
+        old_facts = {f["fact"]: f for f in facts if f["fact"] in self.facts}
+        facts = (list(new_facts.values()) + list(old_facts.values()))[:n]
 
         self.facts.update(f["fact"] for f in facts)
 
         if not facts:
             raise RuntimeError("No vectors found!")
-        else:
-            self.dprint(f"Facts for: {query}", color="yellow")
-            self.dprint(facts)
-            self.dprint.flush("yellow")
+
         return facts
 
     def _answer_question(self, question: str, n: int, classes: list[Classes]) -> Answer:
@@ -253,6 +273,16 @@ class Querier(Chain):
         return {"step": step, "conclusion": conclusion}
 
     def query(self, query: str, n: int = 3, classes: list[Classes] = []):
+        """Runs the entire question-answering process.
+
+        Args:
+            query: The question to ask.
+            n: The number of facts to use from the vector store per query.
+            classes: The interview tags to use as filters (AND).
+
+        Returns:
+            answer (str): The answer to the question.
+        """
         self.dprint(f"Steps for: {query}", color="green")
         steps = self._query(self.steps_template(), "1.", r"\d+(?:\.)", query=query, n=n)
         conclusions = [self._conclude_step(s, query, n, classes) for s in steps]
